/**
 * 扩展MUI的DOM元素对象方法
 * @param {Object} mui
 */
(function(mui) {

	/**
	 * 获取元素属性
	 * @param {Object} name
	 * @param {Object} value
	 */
	mui.fn.attr = function(name, value) {
		if(value === undefined) {
			var value = this[0].getAttribute(name);
			return value;
		} else {
			this[0].setAttribute(name, value);
		}
		return;
	}

	/**
	 * 获取元素附加属性
	 * @param {Object} name
	 * @param {Object} value
	 */
	mui.fn.data = function(name, value) {
		name = 'data-' + name;
		return this.attr(name, value);
	}

	/**
	 * 返回元素value值
	 * @param {Object} value
	 */
	mui.fn.val = function(value) {
		if(value === undefined) {
			return this[0].value;
		} else {
			return this.attr('value', value);
		}
		//		return this.attr('value', value);
	}

	/**
	 * 返回元素html
	 * @param {Object} value
	 */
	mui.fn.html = function(value) {
		if(value === undefined) {
			return this[0].innerHTML;
		} else {
			this[0].innerHTML = value;
		}
		return;
	}
	/**
	 * 绑定事件
	 */
	mui.each(['click', 'tap', 'touch', 'touchStart', 'touchMove', 'blur', 'focus', 'dblclick', 'change', 'submit', 'toggle'], function(index, value) {
//		console.log(value);
		mui.fn[value] = function(fn) {
			if(fn === undefined) {
				this[0][value]();
			} else {
				try{
					return this[0].addEventListener(value, fn);
				} catch(e){
					console.log('aaaaaaaaaaaaa:'+value);
				}
				
			}
		}
	})
})(mui)

/*
 * Common Function by Admin@zhnagp.cn 2017-7-3
 */

//定义网络共用地址
var HOST = "http://cph.buy23.cn/myapi";

/**
 * ajax 网络请求
 * @param {Object} options
 */
function ajax(options, needSign) {
	if(typeof options !== 'object') {
		return false;
	}
	var headers = {
		'Content-Type': 'application/json'
	}
	if(typeof options.headers === 'object') {
		//将传入的header头合并或替换
		mui.each(options.headers, function(key, obj) {
			headers[key] = obj;
		})
	}
	//如果有则附加访问token
	var token = getData("token");
	if(token) {
		if(options.data == undefined) {
			options.data = {};
		}
		options.data.token = token;
	}

	if(needSign) {
		//需要签名
		if(options.data != undefined) {
			var device = getData('device');
			var signString = sign(options.data, device);
			options.data.sign = signString;
		}
	}

	console.log('开始请求：' + HOST + options.url);
	console.log('请求参数:' + JSON.stringify(options.data));
	mui.ajax(HOST + options.url, {
		'async': options.async == false ? false : true,
		'dataType': 'JSON',
		'type': 'post', //强制使用post方式
		'headers': headers,
		'processData': options.processData || true,
		'data': options.data || {}, //
		'timeout': options.timeout || 5000, //五秒不返回数据则触发错误
		'error': options.error || function(res) {
			console.log('请求失败：');
			for(var i in res) {
				console.log(i + '______' + res[i])
			}
		},
		'success': function(res) {
			console.log(options.url + '接口返回：' + JSON.stringify(res))
			res = JSON.parse(res);
			if(res.code) {

				if(res.code == '-1') { // 未登录,提示登录
					if(options.nologin !== undefined){
						options.nologin();
						return ;
					}
					
					mui.confirm('是否前往登录', '请先登录', ['取消', '去登陆'], function(event) {
						if(event.index == 1) { //跳转登录
							plus.io.requestFileSystem(plus.io.PRIVATE_WWW, function(fs) {
								openWindow('file://'+fs.root.fullPath + 'second/login/login.html', {}, {}, true)
							}, function() {})
						} else {
							mui.toast('请先登录');
						}
					}, 'div');
				} else {
					if(typeof options.success == "function") {
						options.success(res.data);
					}
				}
			} else {
				if(typeof options.fail == 'function') {
					options.fail(res.msg);
				} else {
					mui.toast(res.msg);
				}
			}
		},
		'complete': options.complete || function(res) {;
		}
	})
}

/**
 * 拍照或选取照片
 * @param callable callback
 */
function getImg(callback) {
	if(mui.os.plus) {
		var a = [{
			title: "拍照"
		}, {
			title: "从手机相册选择"
		}];
		plus.nativeUI.actionSheet({
			title: "修改用户头像",
			cancel: "取消",
			buttons: a
		}, function(b) { /*actionSheet 按钮点击事件*/
			switch(b.index) {
				case 0:
					break;
				case 1:
					takePhoto(callback); /*拍照*/
					break;
				case 2:
					galleryImg(callback); /*打开相册*/
					break;
				default:
					break;
			}
		})
	}
}

/**
 * 调取摄像头拍照
 * @param callable callback
 */
function takePhoto(callback) {
	var c = plus.camera.getCamera();
	c.captureImage(function(e) {
		plus.io.resolveLocalFileSystemURL(e, function(entry) {
			var s = entry.toLocalURL() + "?version=" + new Date().getTime();
			callback(s); /*回调处理图片*/
		}, function(e) {
			console.log("读取拍照文件错误：" + e.message);
		});
	}, function(s) {
		console.log("error" + s);
	}, {
		filename: "_doc/head.png"
	})
}

/**
 * 从相册选取照片
 * @param callable callback
 */
function galleryImg(callback) {
	//从相册选取一张图片
	plus.gallery.pick(function(a) {
		//根据文件地址，返回一个文件或目录操作对象
		plus.io.resolveLocalFileSystemURL(a, function(entry) {
			plus.io.resolveLocalFileSystemURL("_doc/", function(root) {
				var selectFileName = entry.name;
				root.getFile(selectFileName, {}, function(file) {
					//文件已存在 
					var path = file.fullPath + "?version=" + new Date().getTime();
					callback(path);
					//                  file.remove(function() { 
					//                      entry.copyTo(root, selectFileName, function(e) {
					//                              var path = e.fullPath + "?version=" + new Date().getTime(); 
					//                              callback(path); /*回调处理图片*/
					//                              //变更大图预览的src 
					//                              //目前仅有一张图片，暂时如此处理，后续需要通过标准组件实现 
					//                          }, 
					//                          function(e) { 
					//                              console.log('copy image fail:' + e.message); 
					//                          }); 
					//                  }, function() { 
					//                      console.log("delete image fail:" + e.message);  
					//                  }); 
				}, function() {
					//文件不存在 
					entry.copyTo(root, selectFileName, function(e) {
							var path = e.fullPath + "?version=" + new Date().getTime();
							callback(path); /*回调处理图片*/
						},
						function(e) {
							console.log('copy image fail:' + e.message);
						});
				});
			}, function(e) {
				console.log("get _www folder fail");
			})
		}, function(e) {
			//读取文件或目录操作对象失败
			console.log("读取拍照文件错误：" + e.message);
		});
	}, function(a) {}, {
		filter: "image"
	})
};

/**
 * 设置本地缓存
 * @param string name	数据键
 * @param mixed value 数据内容
 * @param boolean type 是否永久保存（默认关闭会话就失效）
 */
function setData(name, value, type) {
	if(value === undefined) {
		localStorage.removeItem(name), sessionStorage.removeItem(name);
		return;
	}

	if(typeof value == 'object') {
		value = JSON.stringify(value);
	}
	type = type == true;
	if(type) {
		//永久保存
		localStorage.setItem(name, value);
	} else {
		//临时保存
		sessionStorage.setItem(name, value);
	}
	return;
}

/**
 * 设置用户个人缓存
 * @param {Object} name
 * @param {Object} value
 * @param {Object} type
 */
function setUserData(name, value, type) {
	var user = getData('user');
	if(!user) {
		return false;
	} else {
		if(!user.id) {
			return false;
		}
		setData('user_' + user.id + name, value, type);
	}
}
/**
 * 获取本地缓存
 * @param string name
 */
function getData(name) {
	if(name == undefined) {
		var data = {};
		var user = getData('user');
		if(user && user['id']) {
			var start = 'user_' + user['id'].length;
			for(var i in localStorage) {
				var preg = new RegExp("^user_" + user['id']);
				if(/^user_/.test(i) && !preg.test(i)) {
					continue;
				} else {
					data[i] = getData(i);
				}
			}
		} else {
			for(var i in localStorage) {
				if(/^user_/.test(i)) {
					continue;
				}
				data[i] = getData(i);
			}
		}
		for(var i in sessionStorage) {
			data[i] = getData(i);
		}
		return data;
	}
	var data = sessionStorage.getItem(name) || localStorage.getItem(name);
	try {
		var tmpData = JSON.parse(data);
		if(tmpData) {
			return tmpData
		}
	} catch(e) {
		//TODO handle the exception
	}
	return data;
}

/**
 * 获取本地用户缓存
 * @param {Object} name
 */
function getUserData(name) {
	if(name == undefined) {
		return null;
	}
	//	var user = getUser();
	user = getData('user');
	if(!user) {
		return false;
	} else {
		if(!user.id) {
			return false;
		}
		return getData('user_' + user.id + name);
	}
}
/**
 * 清空本地缓存
 */
function clearData(type) {
	type = type == true;
	if(type) {
		localStorage.clear();
	} else {
		sessionStorage.clear();
	}
}

function upload(serverUrl, imgUrl, name, success, error) {
	if(typeof name == 'function') {
		success = name;
		name = undefined;
	}

	name = name || "image";
	if(!/^file\:\/\/\//.test(imgUrl)) {
		imgUrl = "file:///" + imgUrl
	}
	var wt = plus.nativeUI.showWaiting();
	var task = plus.uploader.createUpload(HOST + serverUrl, {
			method: "POST"
		},
		function(t, status) { //上传完成
			if(status == 200) {
				var data = JSON.parse(t.responseText);
				if(data.RETURN == "SUCCESS") {
					if(typeof success == "function") {
						success(data.DATA);
					}
				} else {
					mui.alert('上传失败：' + data.ERROR.errMsg, '上传失败');
					if(typeof error == 'function') {
						error();
					}
				}
				wt.close(); //关闭等待提示按钮
			} else {
				mui.alert("上传失败：" + status, '上传失败');
				if(typeof error == 'function') {
					error();
				}
				wt.close(); //关闭等待提示按钮
			}
		}
	);
	//添加其他参数
	//  task.addData("name","test");
	task.addFile(imgUrl, {
		key: name
	});
	task.start();
}

function checkLogin() {
	var token = getData('token');
	var login = false;
	if(token) {
		//TODO 访问服务器，验证token是否有效
		ajax({
			'url': '/Api/checkLogin',
			'async': false,
			'success': function(res) {
				login = true;
			},
			'fail': function(res) {
				setData('token');
				setData('user');
				login = false;
			}
		})
	} else {
		setData('token');
		openWindow('../login/login.html', {}, {}, true);
		login = false;
	}
	return login
}

function showWindow(win, params) {
	if(typeof win == "string") {
		win = plus.webview.getWebviewById(win)
	}
	if(win == null) {
		return false;
	}
	if(!params) {
		params = {};
	}
	var aniShow = params.aniShow || 'auto';
	var duration = params.time || 300;
	var complete = params.complete || function() {};
	var extras = params.extras || {};
	var data = params.data || {};
	var self = plus.webview.currentWebview();
	plus.webview.show(win, aniShow, duration, complete, extras);
	mui.fire(win, 'show', data);
}

/**
 * 打开新窗口
 * @param {Object} url
 * @param {Object} style
 * @param {Object} extras
 * @param {Object} autoShow
 * @param {Object} params
 */
function openWindow(url, style, extras, autoShow, params) {
	if(!url) {
		return false;
	}
	style = style || {};
	extras = extras || {};
	autoShow = autoShow && true;
	params = params || {};
	var win = plus.webview.create(url, url, style, extras);
	mui.fire(win, 'initPage', params);
	if(autoShow) {
		showWindow(win, params);
	}
	return win;
}

function closeWindow(win) {
	var self = plus.webview.currentWebview();

	if(!win) {
		mui.fire(self, 'closeWindow', {});
		plus.webview.close(self);
	} else {
		if(typeof win == 'string') {
			win = getWindow(win);
		}
		mui.fire(win, 'closeWindow', {});
		plus.webview.close(win);
	}
}

function getWindow(id) {
	if(typeof id == 'string') {
		return plus.webview.getWebviewById(id);
	} else {
		return id;
	}
}

/**
 * 获取用户信息
 */
function getUser() {
	var user = getData('user');
	if(!user) {
		var token = getData('token');
		if(!token) {
			mui.confirm('检查到未登录，是否登录？', '登录提示', ['取消', '去登录'], function(data) {
				if(data.index) {
					checkLogin();
				} else {
					return false;
				}
			}, 'div');
		} else {
			ajax({
				'url': '/user/detailed',
				'async': false,
				'success': function(res) {
					user = res;
				},
				'fail': function(res) {
					return false;
				},
				'error': function() {
					return false;
				}
			})
			return user;
		}
	} else {
		return user;
	}
}

/**
 * 返回带域名的URL
 * @param {Object} url
 */
function url(url) {
	return HOST + url;
}

/**
 * 清理缓存
 */
function clearCache() {
	//TODO 清空缓存，后期完善；
	clearData();
	clearData(true);
	//延时200毫秒后重启应用
	setTimeout(function() {
		plus.runtime.restart();
	}, 200)
}

/**
 * 对比两个对象是否相等
 * @param x
 * @param y
 * @returns {boolean}
 */
function isEqual(a, b) {
	//如果a和b本来就全等
	if(a === b) {
		//判断是否为0和-0
		return a !== 0 || 1 / a === 1 / b;
	}
	//判断是否为null和undefined
	if(a == null || b == null) {
		return a === b;
	}
	//接下来判断a和b的数据类型
	var classNameA = toString.call(a),
		classNameB = toString.call(b);
	//如果数据类型不相等，则返回false
	if(classNameA !== classNameB) {
		return false;
	}
	//如果数据类型相等，再根据不同数据类型分别判断
	switch(classNameA) {
		case '[object RegExp]':
		case '[object String]':
			//进行字符串转换比较
			return '' + a === '' + b;
		case '[object Number]':
			//进行数字转换比较,判断是否为NaN
			if(+a !== +a) {
				return +b !== +b;
			}
			//判断是否为0或-0
			return +a === 0 ? 1 / +a === 1 / b : +a === +b;
		case '[object Date]':
		case '[object Boolean]':
			return +a === +b;
	}
	//如果是对象类型
	if(classNameA == '[object Object]') {
		//获取a和b的属性长度
		var propsA = Object.getOwnPropertyNames(a),
			propsB = Object.getOwnPropertyNames(b);
		if(propsA.length != propsB.length) {
			return false;
		}
		for(var i = 0; i < propsA.length; i++) {
			var propName = propsA[i];
			//如果对应属性对应值不相等，则返回false
			if(a[propName] !== b[propName]) {
				return false;
			}
		}
		return true;
	}
	//如果是数组类型
	if(classNameA == '[object Array]') {
		if(a.toString() == b.toString()) {
			return true;
		}
		return false;
	}
}

function clone(obj) {
	var newObj = Object.create(obj);

	for(var i in obj) {
		if(typeof obj[i] == 'object') {
			newObj[i] = clone(obj[i]);
		} else {
			newObj[i] = obj[i];
		}
	}
	return newObj;
}

/**
 * 随机字符串
 * @param {Object} len
 */
function randomString(len) {　　
	len = len || 32;　　
	var chars = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678'; /****默认去掉了容易混淆的字符oOLl,9gq,Vv,Uu,I1****/ 　　
	var maxPos = chars.length;　　
	var pwd = '';　　
	for(i = 0; i < len; i++) {　　　　
		pwd += chars.charAt(Math.floor(Math.random() * maxPos));　　
	}　　
	return pwd;
}

/**
 * 签名对象排序
 * @param {Object} data
 */
function sign_sort(data) {
	var keyName = [];
	for(var i in data) {
		keyName.push(i);
	}
	keyName.sort();
	var tmpArray = null

	if(data instanceof Array) {
		tmpArray = [];
	} else {
		tmpArray = {};
	}

	for(var i in keyName) {
		if(typeof data[keyName[i]] == 'object') {
			tmpArray[keyName[i]] = sign_sort(data[keyName[i]]);
		} else {
			tmpArray[keyName[i]] = data[keyName[i]];
		}
	}

	return tmpArray;
}

/**
 * 数据签名
 * @param {Object} data
 * @param {Object} device
 */
function sign(data, device) {
	var signArray = sign_sort(data);
	var signString = JSON.stringify(signArray);
	signString += device;

	console.log(signString);
	return hex_md5(signString).toLowerCase();
}

function setDefaultData(data) {
	mui('*[data-default]').each(function(index, obj) {
		var elementName = obj.tagName.toLowerCase();
		var key = obj.dataset.default;
		switch(elementName) {
			case 'input':
				obj.value = data[key];
				break;
			case 'img':
				obj.src = data[key];
				break;
			default:
				obj.innerHTML = data[key];
		}
	})
}

function jump(myPlus) {
	mui('body').on('tap', '.openWindow', function() {
		myPlus.goTo(this.dataset.openurl)
	})
}

function formatDate(times, status, join) {
	
	if(status == undefined){
		status = null
	}
	
	if(join == undefined){
		join = '-';
	}
	
	var now = new Date(parseInt(times * 1000))
	var year = now.getFullYear();
	var month = now.getMonth() + 1;
	var date = now.getDate();
	var hour = now.getHours();
	var minute = now.getMinutes();
	var second = now.getSeconds();

	var time = year + join + month + join + date + " " + hour + ":" + minute + ":" + second;
	if(status == 1) {
		time = year;
	} else if(status == 2) {
		time = year + join + month
	} else if(status == 3) {
		time = year + join + month + join + date
	} else if(status == 4) {
		time = year + join + month + join + date + ' ' + hour
	} else if(status == 5) {
		time = year + join + month + join + date + ' ' + hour + ":" + minute
	}
	return time
}

//百度转gcj坐标
function bdtogcj(bd_lon, bd_lat) {
	var x_pi = 3.14159265358979324 * 3000.0 / 180.0;
	var x = bd_lon - 0.0065;
	var y = bd_lat - 0.006;
	var z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * x_pi);
	var theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * x_pi);
	var gg_lng = (z * Math.cos(theta)).toFixed(7) * 1;
	var gg_lat = (z * Math.sin(theta)).toFixed(7) * 1;
	return [gg_lng, gg_lat]
}
//gcj坐标转百度
function gcjtobd(lng, lat) {
	var x_pi = 3.14159265358979324 * 3000.0 / 180.0;
	var z = Math.sqrt(lng * lng + lat * lat) + 0.00002 * Math.sin(lat * x_pi);
	var theta = Math.atan2(lat, lng) + 0.000003 * Math.cos(lng * x_pi);
	var bd_lng = z * Math.cos(theta) + 0.0065;
	var bd_lat = z * Math.sin(theta) + 0.006;
	return [bd_lng, bd_lat]
}

(function() {
	//检查系统配置相关参数
	var sys = getData('sys');
	if(!sys) {
		ajax({
			'url': '/public/config',
			'success': function(sysData) {
				setData('sys', sysData, true);
			},
		})
	}
})()